@@ -231,11 +231,9 @@ static void * __init probe_security_hook_heads_on_arm64(unsigned int *base)
 		 * Find
 		 *   adrp Xd, #imm21
 		 *   add Xd, Xn, #uimm12
-		 *   add Xd, Xn, #uimm12
 		 * sequence.
-		 * But don't mandate the last one in case it was optimized.
 		 */
-		if ((*ip & 0x98000000) != 0x90000000 ||
+		if ((*ip & 0x9F000000) != 0x90000000 ||
 		    (*(ip + 1) & 0xFFC00000) != 0x91000000)
 			continue;
 		tmp = ((unsigned long) ip) & ~0xFFFUL;
@@ -246,12 +244,20 @@ static void * __init probe_security_hook_heads_on_arm64(unsigned int *base)
 		tmp += offset;
 		offset = (*(ip + 1) >> 10) & 0xFFF;
 		tmp += offset;
-		if ((*(ip + 2) & 0xFFC00000) == 0x91000000) {
-			offset = (*(ip + 2) >> 10) & 0xFFF;
+		/*
+		 * Find
+		 *   ldr Xt, Xn, #uimm12
+		 * sequence.
+		 */
+		for (ip += 2; i < 32 - 2; ip++, i++) {
+			if ((*ip & 0xFFC00000) != 0xF9400000)
+				continue;
+			offset = ((*ip >> 10) & 0xFFF) << 3;
 			tmp += offset;
+			ip4ret = (unsigned int *) tmp;
+			return &ip4ret;
 		}
-		ip4ret = (unsigned int *) tmp;
-		return &ip4ret;
+		break;
 	}
 	return NULL;
 }
